﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using NAudio.Wave;

namespace AY_emulator
{
    public partial class MainForm :Form
    {
        private const float MaxGain = 0.1f;
        private const byte DefaultVolume = 6;
        private const int DefaultTonePeriod = 100;
        private const int DefaultNoisePeriod = 31;
        private const byte DefaultEnvelopeShape = 1;
        private const int DefaultEnvelopePeriod = 1000;

        private MixingWaveProvider32 AYMixer;
        private WasapiOut _player;

        private AYChannel _channelA;
        private AYChannel _channelB;
        private AYChannel _channelC;

        public MainForm()
        {
            InitializeComponent();

            // Set up a mixer
            AYMixer = new MixingWaveProvider32();

            // Set up a player
            _player = new WasapiOut(NAudio.CoreAudioApi.AudioClientShareMode.Shared, true, 30);
            _player.Init(AYMixer);
            _player.Play();


            // Set up the three channels, which will add their signals to the mixer.
            _channelA = new AYChannel(AYMixer);
            _channelB = new AYChannel(AYMixer);
            _channelC = new AYChannel(AYMixer);

            numAVolume.Value = DefaultVolume;
            numBVolume.Value = DefaultVolume;
            numCVolume.Value = DefaultVolume;

            numATonePeriod.Value = DefaultTonePeriod;
            numBTonePeriod.Value = DefaultTonePeriod;
            numCTonePeriod.Value = DefaultTonePeriod;

            _channelA.TonePeriod = (int)numATonePeriod.Value;
            _channelB.TonePeriod = (int)numBTonePeriod.Value;
            _channelC.TonePeriod = (int)numCTonePeriod.Value;

            _channelA.Volume = (byte)numAVolume.Value;
            _channelB.Volume = (byte)numBVolume.Value;
            _channelC.Volume = (byte)numCVolume.Value;

            numANoisePeriod.Value = DefaultNoisePeriod;
            numBNoisePeriod.Value = DefaultNoisePeriod;
            numCNoisePeriod.Value = DefaultNoisePeriod;

            _channelA.NoisePeriod = (int)numANoisePeriod.Value;
            _channelB.NoisePeriod = (int)numBNoisePeriod.Value;
            _channelC.NoisePeriod = (int)numCNoisePeriod.Value;

            numEnvelopeShape.Value = DefaultEnvelopeShape;
            numEnvelopePeriod.Value = DefaultEnvelopePeriod;

            _channelA.EnvelopeShape = (byte)numEnvelopeShape.Value;
            _channelB.EnvelopeShape = (byte)numEnvelopeShape.Value;
            _channelC.EnvelopeShape = (byte)numEnvelopeShape.Value;

            _channelA.EnvelopePeriod = (int)numEnvelopePeriod.Value;
            _channelB.EnvelopePeriod = (int)numEnvelopePeriod.Value;
            _channelC.EnvelopePeriod = (int)numEnvelopePeriod.Value;
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (_player != null)
            {
                if (_player.PlaybackState == PlaybackState.Playing)
                    _player.Stop();
                _player.Dispose();
            }
        }

        #region Channel A
        private void numATonePeriod_ValueChanged(object sender, EventArgs e)
        {
            _channelA.TonePeriod = (int)numATonePeriod.Value;
        }

        private void numANoisePeriod_ValueChanged(object sender, EventArgs e)
        {
            _channelA.NoisePeriod = (int)numANoisePeriod.Value;
        }

        private void chkAToneMix_CheckedChanged(object sender, EventArgs e)
        {
            _channelA.PlayerMix = chkAToneMix.Checked;
        }

        private void chkANoiseMix_CheckedChanged(object sender, EventArgs e)
        {
            _channelA.NoisePlayerMix = chkANoiseMix.Checked;
        }

        private void numAVolume_ValueChanged(object sender, EventArgs e)
        {
            if (chkAEnvelope.Checked)
                _channelA.Volume = (int)numAVolume.Value | 16;
            else
                _channelA.Volume = (int)numAVolume.Value & 15;
        }

        private void chkAEnvelope_CheckedChanged(object sender, EventArgs e)
        {
            if (chkAEnvelope.Checked)
                _channelA.Volume = (int)numAVolume.Value | 16;
            else
                _channelA.Volume = (int)numAVolume.Value & 15;
        }

        private void btnPlayA_Click(object sender, EventArgs e)
        {
            if (chkAEnvelope.Checked)
            {
                _channelA.EnvelopeShape = (byte)numEnvelopeShape.Value;
                _channelA.Volume = (int)numAVolume.Value | 16;
            }

        }
        #endregion
        #region Channel B
        private void numBTonePeriod_ValueChanged(object sender, EventArgs e)
        {
            _channelB.TonePeriod = (int)numBTonePeriod.Value;
        }

        private void numBNoisePeriod_ValueChanged(object sender, EventArgs e)
        {
            _channelB.NoisePeriod = (int)numBNoisePeriod.Value;
        }

        private void chkBToneMix_CheckedChanged(object sender, EventArgs e)
        {
            _channelB.PlayerMix = chkBToneMix.Checked;
        }

        private void chkBNoiseMix_CheckedChanged(object sender, EventArgs e)
        {
            _channelB.NoisePlayerMix = chkBNoiseMix.Checked;
        }

        private void numBVolume_ValueChanged(object sender, EventArgs e)
        {
            if (chkBEnvelope.Checked)
                _channelB.Volume = (int)numBVolume.Value | 16;
            else
                _channelB.Volume = (int)numBVolume.Value & 15;
        }

        private void chkBEnvelope_CheckedChanged(object sender, EventArgs e)
        {
            if (chkBEnvelope.Checked)
                _channelB.Volume = (int)numBVolume.Value | 16;
            else
                _channelB.Volume = (int)numBVolume.Value & 15;
        }

        private void btnPlayB_Click(object sender, EventArgs e)
        {
            if (chkBEnvelope.Checked)
            {
                _channelB.EnvelopeShape = (byte)numEnvelopeShape.Value;
                _channelB.Volume = (int)numBVolume.Value | 16;
            }
        }
        #endregion

        #region Channel C
        private void numCTonePeriod_ValueChanged(object sender, EventArgs e)
        {
            _channelC.TonePeriod = (int)numCTonePeriod.Value;
        }

        private void numCNoisePeriod_ValueChanged(object sender, EventArgs e)
        {
            _channelC.NoisePeriod = (int)numCNoisePeriod.Value;
        }

        private void chkCToneMix_CheckedChanged(object sender, EventArgs e)
        {
            _channelC.PlayerMix = chkCToneMix.Checked;
        }

        private void chkCNoiseMix_CheckedChanged(object sender, EventArgs e)
        {
            _channelC.NoisePlayerMix = chkCNoiseMix.Checked;
        }

        private void numCVolume_ValueChanged(object sender, EventArgs e)
        {
            if (chkCEnvelope.Checked)
                _channelC.Volume = (int)numCVolume.Value | 16;
            else
                _channelC.Volume = (int)numCVolume.Value & 15;
        }

        private void chkCEnvelope_CheckedChanged(object sender, EventArgs e)
        {
            if (chkCEnvelope.Checked)
                _channelC.Volume = (int)numCVolume.Value | 16;
            else
                _channelC.Volume = (int)numCVolume.Value & 15;
        }

        private void btnPlayC_Click(object sender, EventArgs e)
        {
            if (chkCEnvelope.Checked)
            {
                _channelC.EnvelopeShape = (byte)numEnvelopeShape.Value;
                _channelC.Volume = (int)numCVolume.Value | 16;
            }
        }
        #endregion

        #region Envelope
        private void numEnvelopePeriod_ValueChanged(object sender, EventArgs e)
        {
            _channelA.EnvelopePeriod = (int)numEnvelopePeriod.Value;
            _channelB.EnvelopePeriod = (int)numEnvelopePeriod.Value;
        }

        private void numEnvelopeShape_ValueChanged(object sender, EventArgs e)
        {
            _channelA.EnvelopeShape = (byte)numEnvelopeShape.Value;
            _channelB.EnvelopeShape = (byte)numEnvelopeShape.Value;
        }

        #endregion
    }
}
